@@ -222,6 +222,16 @@ g_usb_device_init(GUsbDevice *self)
 	priv->tags = g_ptr_array_new_with_free_func(g_free);
 }
 
+/* private */
+void
+_g_usb_device_add_event(GUsbDevice *self, GUsbDeviceEvent *event)
+{
+	GUsbDevicePrivate *priv = GET_PRIVATE(self);
+	g_return_if_fail(G_USB_IS_DEVICE(self));
+	g_return_if_fail(G_USB_IS_DEVICE_EVENT(event));
+	g_ptr_array_add(priv->events, g_object_ref(event));
+}
+
 gboolean
 _g_usb_device_load(GUsbDevice *self, JsonObject *json_object, GError **error)
 {
@@ -239,6 +249,22 @@ _g_usb_device_load(GUsbDevice *self, JsonObject *json_object, GError **error)
 		g_free(priv->platform_id);
 		priv->platform_id = g_strdup(tmp);
 	}
+	tmp = json_object_get_string_member_with_default(json_object, "Created", NULL);
+	if (tmp != NULL) {
+		g_autoptr(GDateTime) created_new = g_date_time_new_from_iso8601(tmp, NULL);
+		if (created_new == NULL) {
+			g_set_error(error,
+				    G_IO_ERROR,
+				    G_IO_ERROR_INVALID_DATA,
+				    "Cannot parse ISO8601 date: %s",
+				    tmp);
+			return FALSE;
+		}
+		if (!g_date_time_equal(priv->created, created_new)) {
+			g_date_time_unref(priv->created);
+			priv->created = g_steal_pointer(&created_new);
+		}
+	}
 	priv->desc.idVendor = json_object_get_int_member_with_default(json_object, "IdVendor", 0x0);
 	priv->desc.idProduct =
 	    json_object_get_int_member_with_default(json_object, "IdProduct", 0x0);
@@ -450,6 +476,27 @@ _g_usb_device_save(GUsbDevice *self, JsonBuilder *json_builder, GError **error)
 	return TRUE;
 }
 
+/**
+ * g_usb_device_get_created:
+ * @self: a #GUsbDevice
+ *
+ * Gets the date and time that the #GUsbDevice was created.
+ *
+ * This can be used as an indicator if the device replugged, as the vendor and product IDs may not
+ * change for some devices. Use `g_date_time_equal()` to verify equality.
+ *
+ * Returns: (transfer none): a #GDateTime
+ *
+ * Since: 0.4.5
+ **/
+GDateTime *
+g_usb_device_get_created(GUsbDevice *self)
+{
+	GUsbDevicePrivate *priv = GET_PRIVATE(self);
+	g_return_val_if_fail(G_USB_IS_DEVICE(self), NULL);
+	return priv->created;
+}
+
 /**
  * g_usb_device_get_tags:
  * @self: a #GUsbDevice
@@ -824,8 +871,11 @@ g_usb_device_load_event(GUsbDevice *self, const gchar *id)
 	GUsbDevicePrivate *priv = GET_PRIVATE(self);
 
 	/* reset back to the beginning */
-	if (priv->event_idx >= priv->events->len)
+	if (priv->event_idx >= priv->events->len) {
+		if (_g_usb_context_has_flag(priv->context, G_USB_CONTEXT_FLAGS_DEBUG))
+			g_debug("resetting event index");
 		priv->event_idx = 0;
+	}
 
 	/* look for the next event in the sequence */
 	for (guint i = priv->event_idx; i < priv->events->len; i++) {
@@ -919,6 +969,10 @@ g_usb_device_get_custom_index(GUsbDevice *self,
 				    event_id);
 			return 0x00;
 		}
+		if (!g_usb_device_libusb_error_to_gerror(self,
+							 g_usb_device_event_get_rc(event),
+							 error))
+			return 0x00;
 		bytes = g_usb_device_event_get_bytes(event);
 		if (bytes == NULL || g_bytes_get_size(bytes) != 1) {
 			g_set_error(error,
@@ -1116,6 +1170,7 @@ g_usb_device_clear_events(GUsbDevice *self)
 {
 	GUsbDevicePrivate *priv = GET_PRIVATE(self);
 	g_return_if_fail(G_USB_IS_DEVICE(self));
+	priv->event_idx = 0;
 	g_ptr_array_set_size(priv->events, 0);
 }
 
@@ -1571,6 +1626,10 @@ g_usb_device_get_string_descriptor(GUsbDevice *self, guint8 desc_index, GError *
 				    event_id);
 			return NULL;
 		}
+		if (!g_usb_device_libusb_error_to_gerror(self,
+							 g_usb_device_event_get_rc(event),
+							 error))
+			return NULL;
 		bytes = g_usb_device_event_get_bytes(event);
 		if (bytes == NULL) {
 			g_set_error(error,
@@ -1654,6 +1713,10 @@ g_usb_device_get_string_descriptor_bytes_full(GUsbDevice *self,
 				    event_id);
 			return NULL;
 		}
+		if (!g_usb_device_libusb_error_to_gerror(self,
+							 g_usb_device_event_get_rc(event),
+							 error))
+			return 0x00;
 		bytes = g_usb_device_event_get_bytes(event);
 		if (bytes == NULL) {
 			g_set_error(error,
@@ -2159,9 +2222,11 @@ g_usb_device_control_transfer_async(GUsbDevice *self,
 						event_id);
 			return;
 		}
-		if (g_usb_device_event_get_status(event) != LIBUSB_TRANSFER_COMPLETED) {
-			g_usb_device_libusb_status_to_gerror(g_usb_device_event_get_status(event),
-							     &error);
+		if (!g_usb_device_libusb_error_to_gerror(self,
+							 g_usb_device_event_get_rc(event),
+							 &error) ||
+		    !g_usb_device_libusb_status_to_gerror(g_usb_device_event_get_status(event),
+							  &error)) {
 			g_task_report_error(self,
 					    callback,
 					    user_data,
@@ -2243,6 +2308,8 @@ g_usb_device_control_transfer_async(GUsbDevice *self,
 	/* submit transfer */
 	rc = libusb_submit_transfer(req->transfer);
 	if (rc < 0) {
+		if (event != NULL)
+			_g_usb_device_event_set_rc(event, rc);
 		g_usb_device_libusb_error_to_gerror(self, rc, &error);
 		g_task_return_error(task, error);
 		g_object_unref(task);
@@ -2346,9 +2413,11 @@ g_usb_device_bulk_transfer_async(GUsbDevice *self,
 						event_id);
 			return;
 		}
-		if (g_usb_device_event_get_status(event) != LIBUSB_TRANSFER_COMPLETED) {
-			g_usb_device_libusb_status_to_gerror(g_usb_device_event_get_status(event),
-							     &error);
+		if (!g_usb_device_libusb_error_to_gerror(self,
+							 g_usb_device_event_get_rc(event),
+							 &error) ||
+		    !g_usb_device_libusb_status_to_gerror(g_usb_device_event_get_status(event),
+							  &error)) {
 			g_task_report_error(self,
 					    callback,
 					    user_data,
@@ -2419,6 +2488,8 @@ g_usb_device_bulk_transfer_async(GUsbDevice *self,
 	/* submit transfer */
 	rc = libusb_submit_transfer(req->transfer);
 	if (rc < 0) {
+		if (event != NULL)
+			_g_usb_device_event_set_rc(event, rc);
 		g_usb_device_libusb_error_to_gerror(self, rc, &error);
 		g_task_return_error(task, error);
 		g_object_unref(task);
@@ -2522,9 +2593,11 @@ g_usb_device_interrupt_transfer_async(GUsbDevice *self,
 						event_id);
 			return;
 		}
-		if (g_usb_device_event_get_status(event) != LIBUSB_TRANSFER_COMPLETED) {
-			g_usb_device_libusb_status_to_gerror(g_usb_device_event_get_status(event),
-							     &error);
+		if (!g_usb_device_libusb_error_to_gerror(self,
+							 g_usb_device_event_get_rc(event),
+							 &error) ||
+		    !g_usb_device_libusb_status_to_gerror(g_usb_device_event_get_status(event),
+							  &error)) {
 			g_task_report_error(self,
 					    callback,
 					    user_data,
@@ -2595,6 +2668,8 @@ g_usb_device_interrupt_transfer_async(GUsbDevice *self,
 	/* submit transfer */
 	rc = libusb_submit_transfer(req->transfer);
 	if (rc < 0) {
+		if (event != NULL)
+			_g_usb_device_event_set_rc(event, rc);
 		g_usb_device_libusb_error_to_gerror(self, rc, &error);
 		g_task_return_error(task, error);
 		g_object_unref(task);
@@ -2905,21 +2980,42 @@ guint8
 g_usb_device_get_configuration_index(GUsbDevice *self)
 {
 	GUsbDevicePrivate *priv = GET_PRIVATE(self);
+	GUsbDeviceEvent *event = NULL;
 	struct libusb_config_descriptor *config;
 	gint rc;
 	guint8 index;
+	g_autofree gchar *event_id = NULL;
 
 	g_return_val_if_fail(G_USB_IS_DEVICE(self), 0);
 
-	/* sanity check */
-	if (priv->device == NULL)
-		return 0x0;
+	/* build event key either for load or save */
+	if (priv->device == NULL ||
+	    g_usb_context_get_flags(priv->context) & G_USB_CONTEXT_FLAGS_SAVE_EVENTS)
+		event_id = g_strdup_printf("GetConfigurationIndex");
+
+	/* emulated */
+	if (priv->device == NULL) {
+		GBytes *bytes;
+		event = g_usb_device_load_event(self, event_id);
+		if (event == NULL)
+			return 0x0;
+		bytes = g_usb_device_event_get_bytes(event);
+		if (bytes == NULL && g_bytes_get_size(bytes) != 1)
+			return 0x0;
+		return ((const guint8 *) g_bytes_get_data(bytes, NULL))[0];
+	}
 
 	rc = libusb_get_active_config_descriptor(priv->device, &config);
 	g_return_val_if_fail(rc == 0, 0);
 
 	index = config->iConfiguration;
 
+	/* save */
+	if (g_usb_context_get_flags(priv->context) & G_USB_CONTEXT_FLAGS_SAVE_EVENTS) {
+		event = g_usb_device_save_event(self, event_id);
+		_g_usb_device_event_set_bytes_raw(event, &index, sizeof(index));
+	}
+
 	libusb_free_config_descriptor(config);
 	return index;
 }
